@@ -17,7 +17,7 @@ makenode (char *devnode, char *devt)
 	dev = makedev(major, minor);
 	unlink (devnode);
 
-	return mknod(devnode, S_IFCHR | S_IRUSR | S_IWUSR, dev);
+	return mknod(devnode, S_IFBLK | S_IRUSR | S_IWUSR, dev);
 }
 
 extern int
