@@ -6,3 +6,5 @@
 
 #define safe_sprintf(var, format, args...)      \
 	snprintf(var, sizeof(var), format, ##args) >= sizeof(var)
+#define safe_snprintf(var, size, format, args...)      \
+	snprintf(var, size, format, ##args) >= size
