@@ -107,12 +107,10 @@ blacklist (char * dev) {
 	int i;
 	char *p;
 
-	for (i = 0; i < VECTOR_SIZE(conf->blist); i++) {
-		p = VECTOR_SLOT(conf->blist, i);
-
+	vector_foreach_slot (conf->blist, p, i)
 		if (memcmp(dev, p, strlen(p)) == 0)
 			return 1;
-	}
+	
 	return 0;
 }
 
@@ -128,8 +126,7 @@ select_checkfn(struct path *pp)
 	 */
 	pp->checkfn = &readsector0;
 
-	for (i = 0; i < VECTOR_SIZE(conf->hwtable); i++) {
-		hwe = VECTOR_SLOT(conf->hwtable, i);
+	vector_foreach_slot (conf->hwtable, hwe, i) {
 		if (MATCH(hwe->vendor, pp->vendor_id) &&
 		    MATCH(hwe->product, pp->product_id) &&
 		    hwe->checker_index > 0) {
@@ -189,16 +186,14 @@ devinfo (struct path *curpath)
 		curpath->priority = atoi(prio);
 	else {
 		dbg("error calling out %s", buff);
-		curpath->priority = -1;
+		curpath->priority = 1;
 	}
 	dbg("devinfo found prio : %u", curpath->priority);
 
 	/*
 	 * get path uid
 	 */
-	for (i = 0; i < VECTOR_SIZE(conf->hwtable); i++) {
-		hwe = VECTOR_SLOT(conf->hwtable, i);
-
+	vector_foreach_slot (conf->hwtable, hwe, i) {
 		if (MATCH (curpath->vendor_id, hwe->vendor) &&
 		    MATCH (curpath->product_id, hwe->product)) {
 			/*
@@ -373,61 +368,6 @@ get_pathvec_sysfs (vector pathvec)
 	return 0;
 }
 
-static vector
-sort_pathvec_by_prio (vector pathvec)
-{
-	vector sorted_pathvec;
-	struct path * pp;
-	int i;
-	unsigned int highest_prio;
-	unsigned int lowest_prio;
-	unsigned int prio;
-
-	if (VECTOR_SIZE(pathvec) < 2)
-		return pathvec;
-
-	/*
-	 * find out highest & lowest prio
-	 */
-	pp = VECTOR_SLOT(pathvec, 0);
-	highest_prio = pp->priority;
-	lowest_prio = pp->priority;
-
-	for (i = 1; i < VECTOR_SIZE(pathvec); i++) {
-		pp = VECTOR_SLOT(pathvec, i);
-
-		if (pp->priority > highest_prio)
-			highest_prio = pp->priority;
-
-		if (pp->priority < lowest_prio)
-			lowest_prio = pp->priority;
-	}
-
-	if (lowest_prio == highest_prio)
-		return pathvec;
-
-	/* 
-	 * sort
-	 */
-	sorted_pathvec = vector_alloc();
-	dbg("sorted_pathvec :");
-
-	for (prio = lowest_prio; prio <= highest_prio; prio++) {
-		for (i = 0; i < VECTOR_SIZE(pathvec); i++) {
-			pp = VECTOR_SLOT(pathvec, i);
-
-			if (pp->priority == prio) {
-				vector_alloc_slot(sorted_pathvec);
-				vector_set_slot(sorted_pathvec, pp);
-				dbg("%s (%u)", pp->dev, pp->priority);
-			}
-		}
-	}
-
-	return sorted_pathvec;
-
-}
-
 /*
  * print_path style
  */
@@ -464,6 +404,7 @@ print_path (struct path * pp, int style)
 		dbg("undefined path state");
 		break;
 	}
+	printf ("(%s) ", pp->dev_t);
 
 	if (pp->claimed)
 		printf ("[claimed] ");
@@ -486,9 +427,7 @@ print_all_path (vector pathvec)
 
 	fprintf (stdout, "#\n# all paths :\n#\n");
 
-	for (k = 0; k < VECTOR_SIZE(pathvec); k++) {
-		pp = VECTOR_SLOT(pathvec, k);
-		
+	vector_foreach_slot (pathvec, pp, k) {
 		/* leave out paths with incomplete devinfo */
 		if (memcmp (empty_buff, pp->wwid, WWID_SIZE) == 0)
 			continue;
@@ -506,9 +445,7 @@ print_all_mp (vector mp)
 
 	fprintf (stdout, "#\n# all multipaths :\n#\n");
 
-	for (k = 0; k < VECTOR_SIZE(mp); k++) {
-		mpp = VECTOR_SLOT(mp, k);
-
+	vector_foreach_slot (mp, mpp, k) {
 		if (mpp->alias)
 			printf ("%s (%s)", mpp->alias, mpp->wwid);
 		else
@@ -517,10 +454,8 @@ print_all_mp (vector mp)
 		pp = VECTOR_SLOT(mpp->paths, 0);
 		printf (" [%.16s]\n", pp->product_id);
 
-		for (i = 0; i < VECTOR_SIZE(mpp->paths); i++) {
-			pp = VECTOR_SLOT(mpp->paths, i);
+		vector_foreach_slot (mpp->paths, pp, i)
 			print_path (pp, SHORT);
-		}
 	}
 }
 
@@ -530,12 +465,10 @@ find_mp (char * wwid)
 	int i;
 	struct mpentry * mpe;
 
-	for (i = 0; i < VECTOR_SIZE(conf->mptable); i++) {
-                mpe = VECTOR_SLOT(conf->mptable, i);
-
+	vector_foreach_slot (conf->mptable, mpe, i)
                 if (strcmp(mpe->wwid, wwid) == 0)
 			return mpe;
-	}
+
 	return NULL;
 }
 
@@ -552,8 +485,7 @@ coalesce_paths (vector mp, vector pathvec)
 	already_done = 0;
 	memset (empty_buff, 0, WWID_SIZE);
 
-	for (k = 0; k < VECTOR_SIZE(pathvec); k++) {
-		pp1 = VECTOR_SLOT(pathvec, k);
+	vector_foreach_slot (pathvec, pp1, k) {
 		/* skip this path for some reason */
 
 		/* 1. if path has no unique id */
@@ -561,11 +493,10 @@ coalesce_paths (vector mp, vector pathvec)
 			continue;
 
 		/* 2. if mp with this uid already instanciated */
-		for (i = 0; i < VECTOR_SIZE(mp); i++) {
-			mpp = VECTOR_SLOT(mp, i);
+		vector_foreach_slot (mp, mpp, i)
 			if (0 == strcmp (mpp->wwid, pp1->wwid))
 				already_done = 1;
-		}
+
 		if (already_done) {
 			already_done = 0;
 			continue;
@@ -586,7 +517,7 @@ coalesce_paths (vector mp, vector pathvec)
 		vector_alloc_slot (mpp->paths);
 		vector_set_slot (mpp->paths, VECTOR_SLOT(pathvec, k));
 
-		if (mpp->size == 0)
+		if (mpp->size == 0 && pp1->state == PATH_UP)
 			mpp->size = get_disk_size(pp1->dev);
 
 		for (i = k + 1; i < VECTOR_SIZE(pathvec); i++) {
@@ -595,6 +526,10 @@ coalesce_paths (vector mp, vector pathvec)
 				vector_alloc_slot(mpp->paths);
 				vector_set_slot(mpp->paths,
 						VECTOR_SLOT(pathvec, i));
+
+				if (mpp->size == 0 && pp2->state == PATH_UP)
+					mpp->size = get_disk_size(pp2->dev);
+
 			}
 		}
 		vector_set_slot(mp, mpp);
@@ -662,7 +597,7 @@ dm_map_present (char * str)
 	}
 
 	do {
-		if (0 == strcmp(names->name, str))
+		if (0 == strncmp(names->name, str, strlen(names->name)))
 			r = 1;
 		next = names->next;
 		names = (void *) names + next;
@@ -769,9 +704,7 @@ select_selector (struct multipath * mp)
 	mp->selector_args = conf->default_selector_args;
 
 	/* 2) override by controler wide settings */
-	for (i = 0; i < VECTOR_SIZE(conf->hwtable); i++) {
-		hwe = VECTOR_SLOT(conf->hwtable, i);
-
+	vector_foreach_slot (conf->hwtable, hwe, i) {
 		if (strcmp(hwe->vendor, pp->vendor_id) == 0 &&
 		    strcmp(hwe->product, pp->product_id) == 0) {
 			mp->selector = (hwe->selector) ?
@@ -782,9 +715,7 @@ select_selector (struct multipath * mp)
 	}
 
 	/* 3) override by LUN specific settings */
-	for (i = 0; i < VECTOR_SIZE(conf->mptable); i++) {
-		mpe = VECTOR_SLOT(conf->mptable, i);
-
+	vector_foreach_slot (conf->mptable, mpe, i) {
 		if (strcmp(mpe->wwid, mp->wwid) == 0) {
 			mp->selector = (mpe->selector) ?
 				   mpe->selector : conf->default_selector;
@@ -830,8 +761,7 @@ assemble_map (struct multipath * mp)
 		p += shift;
 		freechar -= shift;
 
-		for (j = 0; j < VECTOR_SIZE(pgpaths); j++) {
-			pp = VECTOR_SLOT(pgpaths, j);
+		vector_foreach_slot (pgpaths, pp, j) {
 			shift = snprintf(p, freechar, " %s", pp->dev_t);
 			if (shift >= freechar) {
 				fprintf(stderr, "mp->params too small\n");
@@ -871,12 +801,10 @@ setup_map (vector pathvec, struct multipath * mpp)
 	 * don't bother if a constituant path is claimed
 	 * FIXME : claimed detection broken, always unclaimed for now
 	 */
-	for (i = 0; i < VECTOR_SIZE(mpp->paths); i++) {
-		pp = VECTOR_SLOT(mpp->paths, i);
-
+	vector_foreach_slot (mpp->paths, pp, i)
 		if (pp->claimed)
 			return 0;
-	}
+
 	pp = VECTOR_SLOT(mpp->paths, 0);
 
 	/*
@@ -909,9 +837,7 @@ setup_map (vector pathvec, struct multipath * mpp)
 	/*
 	 * 3) override by controler wide setting
 	 */
-	for (i = 0; i < VECTOR_SIZE(conf->hwtable); i++) {
-		hwe = VECTOR_SLOT(conf->hwtable, i);
-
+	vector_foreach_slot (conf->hwtable, hwe, i) {
 		if (MATCH (pp->vendor_id, hwe->vendor) &&
 		    MATCH (pp->product_id, hwe->product) &&
 		    hwe->iopolicy > 0) {
@@ -924,9 +850,7 @@ setup_map (vector pathvec, struct multipath * mpp)
 	/*
 	 * 4) override by LUN specific setting
 	 */
-	for (i = 0; i < VECTOR_SIZE(conf->mptable); i++) {
-		mpe = VECTOR_SLOT(conf->mptable, i);
-
+	vector_foreach_slot (conf->mptable, mpe, i) {
 		if (strcmp(mpe->wwid, mpp->wwid) == 0 &&
 		    mpe->iopolicy > 0) {
 			iopolicy = mpe->iopolicy;
@@ -951,26 +875,20 @@ setup_map (vector pathvec, struct multipath * mpp)
 
 	/*
 	 * layered map computation :
-	 *  1) sort paths by priority
-	 *  2) separate failed paths in a tersary PG
-	 *  3) separate shaky paths in a secondary PG
-	 *  4) apply selected grouping policy to valid paths
-	 *  5) reorder path groups by summed priority
+	 *  1) separate failed paths in a tersary PG
+	 *  2) separate shaky paths in a secondary PG
+	 *  3) apply selected grouping policy to valid paths
+	 *  4) reorder path groups by summed priority
 	 */
 
 	/*
-	 * 1) sort by priority
-	 */
-	mpp->paths = sort_pathvec_by_prio(mpp->paths);
-
-	/*
-	 * 2) & 3)
+	 * 1) & 2)
 	 */
 	group_by_status(mpp, PATH_DOWN);
 	group_by_status(mpp, PATH_SHAKY);
 
 	/*
-	 * 4) apply selected grouping policy
+	 * 3) apply selected grouping policy
 	 */
 	if (iopolicy == MULTIBUS)
 		one_group (mpp);
@@ -990,7 +908,7 @@ setup_map (vector pathvec, struct multipath * mpp)
 	}
 
 	/*
-	 * 5)
+	 * 4)
 	 */
 	sort_pg_by_summed_prio(mpp);
 
@@ -1175,7 +1093,7 @@ usage (char * progname)
 		"\t   1\t\t\tprint created devmap names only\n" \
 		"\t   2\t\t\tprint all paths and multipaths\n" \
 		"\t-d\t\tdry run, do not create or update devmaps\n" \
-		"\t-D maj min\tlimit scope to the device's multipath\n" \
+		"\t-D maj:min\tlimit scope to the device's multipath\n" \
 		"\t\t\t(major:minor device reference)\n"
 		"\t-S\t\tinhibit signal sending to multipathd\n"
 		"\n" \
@@ -1375,9 +1293,7 @@ main (int argc, char *argv[])
 	 * may be conf->dev is a mapname
 	 * if so, only reconfigure this map
 	 */
-	for (k = 0; k < VECTOR_SIZE(mp); k++) {
-		mpp = VECTOR_SLOT(mp, k);
-
+	vector_foreach_slot (mp, mpp, k) {
 		if (conf->dev && (
 		    (mpp->alias &&
 		    0 == strncmp(mpp->alias, conf->dev, FILE_NAME_SIZE)) ||
@@ -1395,10 +1311,8 @@ main (int argc, char *argv[])
 	if (conf->verbosity > 1)
 		fprintf(stdout, "#\n# device maps :\n#\n");
 
-	for (k = 0; k < VECTOR_SIZE(mp); k++) {
-		mpp = VECTOR_SLOT(mp, k);
+	vector_foreach_slot (mp, mpp, k)
 		setup_map(pathvec, mpp);
-	}
 
 out:
 	/*
